#!/usr/bin/env node

'use strict';

const program = require('commander');
const packageInfo = require('../../package.json');
const Utils = require('../utils');

let subCmdValue = '';

program
    .version(packageInfo.version, '-v, --version')
    .usage('<command>');

program.command('watch')
    .description('监听文件')
    .action(function () {
        subCmdValue = 'watch';
    });
program.command('build')
    .description('构建文件')
    .action(function () {
        subCmdValue = 'build';
    });
program.command('server')
    .description('开发服务')
    .action(function () {
        subCmdValue = 'server';
    });
program.command('publish')
    .description('发布文件')
    .action(function () {
        subCmdValue = 'publish';
    });
program.command('init')
    .description('新建项目')
    .action(function () {
        subCmdValue = 'init';
    });

program.on('--help', function () {
    console.log(program.name() + '@' + program.version());
    console.log('');
});

program.parse(process.argv);

process.env['GUIDO_MODE'] = subCmdValue;

if (subCmdValue === 'watch' || subCmdValue === 'build' || subCmdValue === 'publish' || subCmdValue === 'server') {
    let isProduction = subCmdValue === 'publish';
    let isWatch = subCmdValue === 'watch';
    let isServer = subCmdValue === 'server';
    let env = isProduction ? 'production' : 'development';
    process.env['GUIDO_ENV'] = env;

    const chalk = require('chalk');

    const build = require('../build');
    build({
        env: env,
        watch: isWatch,
        devServer: isServer
    }, function (err, stats, webpackStateConfig) {
        switch (subCmdValue) {
            case 'build':
            case 'publish':
                Utils.printLog(chalk.green('Compiler Complete'));
                Utils.printLog(chalk.green('Stats'));
                Utils.printLog(stats.toString(webpackStateConfig), {
                    prefix: ''
                });

                if (stats.hasErrors()) {
                    process.on('exit', function () {
                        process.exit(1);
                    });
                } else {
                    process.exit(0);
                }
                break;
            case 'watch':
                Utils.printLog(chalk.green('Watch mode start'));
                Utils.printLog(chalk.green('Stats'));
                Utils.printLog(stats.toString(webpackStateConfig), {
                    prefix: ''
                });
                break;
        }
    });
} else if (subCmdValue === 'init') {
    const init = require('../init');
    init({}, function () {
        process.exit(0);
    });
} else {
    program.help();
}
